@@ -17,8 +17,8 @@
 # FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU Lesser General Public License along
-# with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Classes for geometry operations"""
 
@@ -29,9 +29,11 @@ import hashlib
 
 import numpy as np
 import yaml
-from pyproj import Geod, Proj
+from pyproj import Geod
 
-from pyresample import _spatial_mp, utils, CHUNK_SIZE
+from pyresample import utils, CHUNK_SIZE
+from pyresample._spatial_mp import Proj, Proj_MP, Cartesian, Cartesian_MP
+from pyresample.boundary import AreaDefBoundary, Boundary, SimpleBoundary
 
 try:
     from xarray import DataArray
@@ -49,21 +51,8 @@ class IncompatibleAreas(ValueError):
     """Error when the areas to combine are not compatible."""
 
 
-class Boundary(object):
-
-    """Container for geometry boundary.
-    Labelling starts in upper left corner and proceeds clockwise"""
-
-    def __init__(self, side1, side2, side3, side4):
-        self.side1 = side1
-        self.side2 = side2
-        self.side3 = side3
-        self.side4 = side4
-
-
 class BaseDefinition(object):
-
-    """Base class for geometry definitions
+    """Base class for geometry definitions.
 
     .. versionchanged:: 1.8.0
 
@@ -90,6 +79,22 @@ class BaseDefinition(object):
         self.lons = lons
         self.ndim = None
         self.cartesian_coords = None
+        self.hash = None
+
+    def __getitem__(self, key):
+        """Slice a 2D geographic definition."""
+        y_slice, x_slice = key
+        return self.__class__(
+            lons=self.lons[y_slice, x_slice],
+            lats=self.lats[y_slice, x_slice],
+            nprocs=self.nprocs
+        )
+
+    def __hash__(self):
+        """Compute the hash of this object."""
+        if self.hash is None:
+            self.hash = int(self.update_hash().hexdigest(), 16)
+        return self.hash
 
     def __eq__(self, other):
         """Test for approximate equality"""
@@ -190,15 +195,40 @@ class BaseDefinition(object):
         else:
             return self.lons[data_slice], self.lats[data_slice]
 
-    def get_boundary_lonlats(self):
-        """Returns Boundary objects"""
+    def get_lonlats_dask(self, chunks=CHUNK_SIZE):
+        """Get the lon lats as a single dask array."""
+        import dask.array as da
+        lons, lats = self.get_lonlats()
 
-        side1 = self.get_lonlats(data_slice=(0, slice(None)))
-        side2 = self.get_lonlats(data_slice=(slice(None), -1))
-        side3 = self.get_lonlats(data_slice=(-1, slice(None)))
-        side4 = self.get_lonlats(data_slice=(slice(None), 0))
-        return (Boundary(side1[0], side2[0], side3[0][::-1], side4[0][::-1]),
-                Boundary(side1[1], side2[1], side3[1][::-1], side4[1][::-1]))
+        if isinstance(lons.data, da.Array):
+            return lons.data, lats.data
+        else:
+            lons = da.from_array(np.asanyarray(lons),
+                                 chunks=chunks)
+            lats = da.from_array(np.asanyarray(lats),
+                                 chunks=chunks)
+        return lons, lats
+
+    def get_boundary_lonlats(self):
+        """Return Boundary objects."""
+        s1_lon, s1_lat = self.get_lonlats(data_slice=(0, slice(None)))
+        s2_lon, s2_lat = self.get_lonlats(data_slice=(slice(None), -1))
+        s3_lon, s3_lat = self.get_lonlats(data_slice=(-1, slice(None, None, -1)))
+        s4_lon, s4_lat = self.get_lonlats(data_slice=(slice(None, None, -1), 0))
+        return (SimpleBoundary(s1_lon.squeeze(), s2_lon.squeeze(), s3_lon.squeeze(), s4_lon.squeeze()),
+                SimpleBoundary(s1_lat.squeeze(), s2_lat.squeeze(), s3_lat.squeeze(), s4_lat.squeeze()))
+
+    def get_bbox_lonlats(self):
+        """Returns the bounding box lons and lats"""
+
+        s1_lon, s1_lat = self.get_lonlats(data_slice=(0, slice(None)))
+        s2_lon, s2_lat = self.get_lonlats(data_slice=(slice(None), -1))
+        s3_lon, s3_lat = self.get_lonlats(data_slice=(-1, slice(None, None, -1)))
+        s4_lon, s4_lat = self.get_lonlats(data_slice=(slice(None, None, -1), 0))
+        return zip(*[(s1_lon.squeeze(), s1_lat.squeeze()),
+                     (s2_lon.squeeze(), s2_lat.squeeze()),
+                     (s3_lon.squeeze(), s3_lat.squeeze()),
+                     (s4_lon.squeeze(), s4_lat.squeeze())])
 
     def get_cartesian_coords(self, nprocs=None, data_slice=None, cache=False):
         """Retrieve cartesian coordinates of geometry definition
@@ -230,9 +260,9 @@ class BaseDefinition(object):
             lons, lats = self.get_lonlats(nprocs=nprocs, data_slice=data_slice)
 
             if nprocs > 1:
-                cartesian = _spatial_mp.Cartesian_MP(nprocs)
+                cartesian = Cartesian_MP(nprocs)
             else:
-                cartesian = _spatial_mp.Cartesian()
+                cartesian = Cartesian()
 
             cartesian_coords = cartesian.transform_lonlats(np.ravel(lons),
                                                            np.ravel(lats))
@@ -361,6 +391,10 @@ class BaseDefinition(object):
         inter_area = get_polygon_area(self.intersection(other))
         return inter_area / other_area
 
+    def get_area_slices(self, area_to_cover):
+        """Compute the slice to read based on an `area_to_cover`."""
+        raise NotImplementedError
+
 
 class CoordinateDefinition(BaseDefinition):
     """Base class for geometry definitions defined by lons and lats only"""
@@ -491,36 +525,23 @@ class SwathDefinition(CoordinateDefinition):
         elif lons.ndim > 2:
             raise ValueError('Only 1 and 2 dimensional swaths are allowed')
 
-        self.hash = None
-
     def __hash__(self):
         """Compute the hash of this object."""
         if self.hash is None:
-            hasher = hashlib.sha1()
-            hasher.update(get_array_hashable(self.lons))
-            hasher.update(get_array_hashable(self.lats))
-            try:
-                if self.lons.mask is not np.bool_(False):
-                    hasher.update(get_array_hashable(self.lons.mask))
-            except AttributeError:
-                pass
-            self.hash = int(hasher.hexdigest(), 16)
-
+            self.hash = int(self.update_hash().hexdigest(), 16)
         return self.hash
 
-    def get_lonlats_dask(self, chunks=CHUNK_SIZE):
-        """Get the lon lats as a single dask array."""
-        import dask.array as da
-        lons, lats = self.get_lonlats()
-
-        if isinstance(lons.data, da.Array):
-            return lons.data, lats.data
-        else:
-            lons = da.from_array(np.asanyarray(lons),
-                                 chunks=chunks)
-            lats = da.from_array(np.asanyarray(lats),
-                                 chunks=chunks)
-        return lons, lats
+    def update_hash(self, the_hash=None):
+        if the_hash is None:
+            the_hash = hashlib.sha1()
+        the_hash.update(get_array_hashable(self.lons))
+        the_hash.update(get_array_hashable(self.lats))
+        try:
+            if self.lons.mask is not np.bool_(False):
+                the_hash.update(get_array_hashable(self.lons.mask))
+        except AttributeError:
+            pass
+        return the_hash
 
     def _compute_omerc_parameters(self, ellipsoid):
         """Compute the oblique mercator projection bouding box parameters."""
@@ -531,14 +552,33 @@ class SwathDefinition(CoordinateDefinition):
 
         proj_dict2points = {'proj': 'omerc', 'lat_0': lat, 'ellps': ellipsoid,
                             'lat_1': lat1, 'lon_1': lon1,
-                            'lat_2': lat2, 'lon_2': lon2}
+                            'lat_2': lat2, 'lon_2': lon2,
+                            'no_rot': True
+                            }
 
+        # We need to compute alpha-based omerc for geotiff support
         lonc, lat0 = Proj(**proj_dict2points)(0, 0, inverse=True)
+        az1, az2, dist = Geod(**proj_dict2points).inv(lonc, lat0, lon2, lat2)
+        azimuth = az1
         az1, az2, dist = Geod(**proj_dict2points).inv(lonc, lat0, lon1, lat1)
-        del az2, dist
-        return {'proj': 'omerc', 'alpha': float(az1),
-                'lat_0': float(lat0),  'lonc': float(lonc),
-                'no_rot': True, 'ellps': ellipsoid}
+        if abs(az1 - azimuth) > 1:
+            if abs(az2 - azimuth) > 1:
+                logger.warning("Can't find appropriate azimuth.")
+            else:
+                azimuth += az2
+                azimuth /= 2
+        else:
+            azimuth += az1
+            azimuth /= 2
+        if abs(azimuth) > 90:
+            azimuth = 180 + azimuth
+
+        prj_params = {'proj': 'omerc', 'alpha': float(azimuth),
+                      'lat_0': float(lat0),  'lonc': float(lonc),
+                      'gamma': 0,
+                      'ellps': ellipsoid}
+
+        return prj_params
 
     def _compute_generic_parameters(self, projection, ellipsoid):
         """Compute the projection bb parameters for most projections."""
@@ -550,11 +590,9 @@ class SwathDefinition(CoordinateDefinition):
 
     def get_edge_lonlats(self):
         """Get the concatenated boundary of the current swath."""
-        lons, lats = self.get_boundary_lonlats()
-        blons = np.ma.concatenate([lons.side1, lons.side2,
-                                   lons.side3, lons.side4])
-        blats = np.ma.concatenate([lats.side1, lats.side2,
-                                   lats.side3, lats.side4])
+        lons, lats = self.get_bbox_lonlats()
+        blons = np.ma.concatenate(lons)
+        blats = np.ma.concatenate(lats)
         return blons, blats
 
     def compute_bb_proj_params(self, proj_dict):
@@ -586,9 +624,6 @@ class SwathDefinition(CoordinateDefinition):
 
         proj_dict = self.compute_bb_proj_params(proj_dict)
 
-        if projection == 'omerc':
-            x_size, y_size = y_size, x_size
-
         area = DynamicAreaDefinition(area_id, description, proj_dict)
         lons, lats = self.get_edge_lonlats()
         return area.freeze((lons, lats), size=(x_size, y_size))
@@ -803,6 +838,7 @@ class AreaDefinition(BaseDefinition):
         self.x_size = int(x_size)
         self.y_size = int(y_size)
         self.shape = (y_size, x_size)
+        self.crop_offset = (0, 0)
         try:
             self.rotation = float(rotation)
         except TypeError:
@@ -819,7 +855,7 @@ class AreaDefinition(BaseDefinition):
         self.area_extent = tuple(area_extent)
 
         # Calculate area_extent in lon lat
-        proj = _spatial_mp.Proj(**proj_dict)
+        proj = Proj(**proj_dict)
         corner_lons, corner_lats = proj((area_extent[0], area_extent[2]),
                                         (area_extent[1], area_extent[3]),
                                         inverse=True)
@@ -843,6 +879,12 @@ class AreaDefinition(BaseDefinition):
 
         self.dtype = dtype
 
+    def __hash__(self):
+        """Compute the hash of this object."""
+        if self.hash is None:
+            self.hash = int(self.update_hash().hexdigest(), 16)
+        return self.hash
+
     @property
     def proj_str(self):
         return utils.proj4_dict_to_str(self.proj_dict, sort=True)
@@ -866,6 +908,17 @@ class AreaDefinition(BaseDefinition):
                                            proj_str, self.x_size, self.y_size,
                                            self.area_extent)
 
+    __repr__ = __str__
+
+    def to_cartopy_crs(self):
+        from pyresample._cartopy import from_proj
+        bounds = (self.area_extent[0],
+                  self.area_extent[2],
+                  self.area_extent[1],
+                  self.area_extent[3])
+        crs = from_proj(self.proj_str, bounds=bounds)
+        return crs
+
     def create_areas_def(self):
         to_dump = OrderedDict()
         res = OrderedDict()
@@ -902,8 +955,6 @@ class AreaDefinition(BaseDefinition):
                                   area_extent=self.area_extent)
         return area_def_str
 
-    __repr__ = __str__
-
     def __eq__(self, other):
         """Test for equality"""
 
@@ -919,12 +970,14 @@ class AreaDefinition(BaseDefinition):
 
         return not self.__eq__(other)
 
-    def __hash__(self):
-        return hash((
-            self.proj_str,
-            self.shape,
-            self.area_extent
-        ))
+    def update_hash(self, the_hash=None):
+        """Update a hash, or return a new one if needed."""
+        if the_hash is None:
+            the_hash = hashlib.sha1()
+        the_hash.update(self.proj_str.encode('utf-8'))
+        the_hash.update(np.array(self.shape))
+        the_hash.update(np.array(self.area_extent))
+        return the_hash
 
     def colrow2lonlat(self, cols, rows):
         """
@@ -933,7 +986,7 @@ class AreaDefinition(BaseDefinition):
         To be used with scarse data points instead of slices
         (see get_lonlats).
         """
-        p = _spatial_mp.Proj(self.proj4_string)
+        p = Proj(self.proj_str)
         x = self.projection_x_coords
         y = self.projection_y_coords
         return p(y[y.size - cols], x[x.size - rows], inverse=True)
@@ -979,7 +1032,7 @@ class AreaDefinition(BaseDefinition):
             if lon.shape != lat.shape:
                 raise ValueError("lon and lat is not of the same shape!")
 
-        pobj = _spatial_mp.Proj(self.proj4_string)
+        pobj = Proj(self.proj_str)
         xm_, ym_ = pobj(lon, lat)
 
         return self.get_xy_from_proj_coords(xm_, ym_)
@@ -1033,15 +1086,15 @@ class AreaDefinition(BaseDefinition):
         y__ = (ym - upl_y) / yscale
 
         if isinstance(x__, np.ndarray) and isinstance(y__, np.ndarray):
-            mask = (((x__ < 0) | (x__ > self.x_size)) |
-                    ((y__ < 0) | (y__ > self.y_size)))
+            mask = (((x__ < 0) | (x__ >= self.x_size)) |
+                    ((y__ < 0) | (y__ >= self.y_size)))
             return (np.ma.masked_array(x__.astype('int'), mask=mask,
                                        fill_value=-1, copy=False),
                     np.ma.masked_array(y__.astype('int'), mask=mask,
                                        fill_value=-1, copy=False))
         else:
-            if ((x__ < 0 or x__ > self.x_size) or
-                    (y__ < 0 or y__ > self.y_size)):
+            if ((x__ < 0 or x__ >= self.x_size) or
+                    (y__ < 0 or y__ >= self.y_size)):
                 raise ValueError('Point outside area:( %f %f)' % (x__, y__))
             return int(x__), int(y__)
 
@@ -1058,7 +1111,8 @@ class AreaDefinition(BaseDefinition):
         (lon, lat) : tuple of floats
         """
 
-        return self.get_lonlats(nprocs=None, data_slice=(row, col))
+        lon, lat = self.get_lonlats(nprocs=None, data_slice=(row, col))
+        return np.asscalar(lon), np.asscalar(lat)
 
     def get_proj_vectors_dask(self, chunks=CHUNK_SIZE, dtype=None):
         import dask.array as da
@@ -1131,60 +1185,16 @@ class AreaDefinition(BaseDefinition):
         if dtype is None:
             dtype = self.dtype
 
-        # create coordinates of local area as ndarrays
+        target_x = np.arange(self.x_size, dtype=dtype) * self.pixel_size_x + self.pixel_upper_left[0]
+        target_y = np.arange(self.y_size, dtype=dtype) * -self.pixel_size_y + self.pixel_upper_left[1]
         if data_slice is None or data_slice == slice(None):
-            # Full slice
-            rows = self.y_size
-            cols = self.x_size
-            row_start = 0
-            col_start = 0
+            pass
+        elif isinstance(data_slice, slice):
+            target_y = target_y[data_slice]
         else:
-            if isinstance(data_slice, slice):
-                # Row slice
-                row_start = get_val(data_slice.start, 0, self.y_size)
-                col_start = 0
-                rows = get_val(
-                    data_slice.stop, self.y_size, self.y_size) - row_start
-                cols = self.x_size
-            elif isinstance(data_slice[0], slice) and isinstance(data_slice[1], slice):
-                # Block slice
-                row_start = get_val(data_slice[0].start, 0, self.y_size)
-                col_start = get_val(data_slice[1].start, 0, self.x_size)
-                rows = get_val(
-                    data_slice[0].stop, self.y_size, self.y_size) - row_start
-                cols = get_val(
-                    data_slice[1].stop, self.x_size, self.x_size) - col_start
-            elif isinstance(data_slice[0], slice):
-                # Select from col
-                is_1d_select = True
-                row_start = get_val(data_slice[0].start, 0, self.y_size)
-                col_start = get_val(data_slice[1], 0, self.x_size)
-                rows = get_val(
-                    data_slice[0].stop, self.y_size, self.y_size) - row_start
-                cols = 1
-            elif isinstance(data_slice[1], slice):
-                # Select from row
-                is_1d_select = True
-                row_start = get_val(data_slice[0], 0, self.y_size)
-                col_start = get_val(data_slice[1].start, 0, self.x_size)
-                rows = 1
-                cols = get_val(
-                    data_slice[1].stop, self.x_size, self.x_size) - col_start
-            else:
-                # Single element select
-                is_single_value = True
+            target_y = target_y[data_slice[0]]
+            target_x = target_x[data_slice[1]]
 
-                row_start = get_val(data_slice[0], 0, self.y_size)
-                col_start = get_val(data_slice[1], 0, self.x_size)
-
-                rows = 1
-                cols = 1
-
-        # Calculate coordinates
-        target_x = np.arange(col_start, col_start + cols, dtype=dtype) * \
-            self.pixel_size_x + self.pixel_upper_left[0]
-        target_y = np.arange(row_start, row_start + rows, dtype=dtype) * - \
-            self.pixel_size_y + self.pixel_upper_left[1]
         if self.rotation != 0:
             res = do_rotation(target_x, target_y, self.rotation)
             target_x, target_y = res[0, :, :], res[1, :, :]
@@ -1209,11 +1219,11 @@ class AreaDefinition(BaseDefinition):
 
     @property
     def projection_x_coords(self):
-        return self.get_proj_coords(data_slice=(0, slice(None)))[0]
+        return self.get_proj_coords(data_slice=(0, slice(None)))[0].squeeze()
 
     @property
     def projection_y_coords(self):
-        return self.get_proj_coords(data_slice=(slice(None), 0))[1]
+        return self.get_proj_coords(data_slice=(slice(None), 0))[1].squeeze()
 
     @property
     def proj_x_coords(self):
@@ -1229,10 +1239,10 @@ class AreaDefinition(BaseDefinition):
 
     @property
     def outer_boundary_corners(self):
-        """Returns the lon,lat of the outer edges of the corner points
+        """Return the lon,lat of the outer edges of the corner points
         """
         from pyresample.spherical_geometry import Coordinate
-        proj = _spatial_mp.Proj(**self.proj_dict)
+        proj = Proj(**self.proj_dict)
 
         corner_lons, corner_lats = proj((self.area_extent[0], self.area_extent[2],
                                          self.area_extent[2], self.area_extent[0]),
@@ -1257,7 +1267,7 @@ class AreaDefinition(BaseDefinition):
         return res[:, :, 0], res[:, :, 1]
 
     def get_lonlats(self, nprocs=None, data_slice=None, cache=False, dtype=None):
-        """Returns lon and lat arrays of area.
+        """Return lon and lat arrays of area.
 
         Parameters
         ----------
@@ -1285,9 +1295,9 @@ class AreaDefinition(BaseDefinition):
 
             # Proj.4 definition of target area projection
             if nprocs > 1:
-                target_proj = _spatial_mp.Proj_MP(**self.proj_dict)
+                target_proj = Proj_MP(**self.proj_dict)
             else:
-                target_proj = _spatial_mp.Proj(**self.proj_dict)
+                target_proj = Proj(**self.proj_dict)
 
             # Get coordinates of local area as ndarrays
             target_x, target_y = self.get_proj_coords(
@@ -1321,10 +1331,123 @@ class AreaDefinition(BaseDefinition):
 
     @property
     def proj4_string(self):
-        """Returns projection definition as Proj.4 string"""
+        """Return projection definition as Proj.4 string."""
+        warnings.warn("'proj4_string' is deprecated, please use 'proj_str' "
+                      "instead.", DeprecationWarning)
+        return utils.proj4_dict_to_str(self.proj_dict)
 
-        items = self.proj_dict.items()
-        return '+' + ' +'.join([t[0] + '=' + str(t[1]) for t in items])
+    def get_area_slices(self, area_to_cover):
+        """Compute the slice to read based on an `area_to_cover`."""
+        if not isinstance(area_to_cover, AreaDefinition):
+            raise NotImplementedError('Only AreaDefinitions can be used')
+
+        # Intersection only required for two different projections
+        if area_to_cover.proj_str == self.proj_str:
+            logger.debug('Projections for data and slice areas are'
+                         ' identical: %s',
+                         area_to_cover.proj_dict.get('proj', area_to_cover.proj_dict.get('init')))
+            # Get xy coordinates
+            llx, lly, urx, ury = area_to_cover.area_extent
+            x, y = self.get_xy_from_proj_coords([llx, urx], [lly, ury])
+
+            xstart = 0 if x[0] is np.ma.masked else x[0]
+            ystart = 0 if y[1] is np.ma.masked else y[1]
+            xstop = self.x_size if x[1] is np.ma.masked else x[1] + 1
+            ystop = self.y_size if y[0] is np.ma.masked else y[0] + 1
+
+            return slice(xstart, xstop), slice(ystart, ystop)
+
+        if self.proj_dict.get('proj') != 'geos':
+            raise NotImplementedError("Source projection must be 'geos' if "
+                                      "source/target projections are not "
+                                      "equal.")
+
+        data_boundary = Boundary(*get_geostationary_bounding_box(self))
+        if area_to_cover.proj_dict.get('proj') == 'geos':
+            area_boundary = Boundary(
+                *get_geostationary_bounding_box(area_to_cover))
+        else:
+            area_boundary = AreaDefBoundary(area_to_cover, 100)
+
+        intersection = data_boundary.contour_poly.intersection(
+            area_boundary.contour_poly)
+        if intersection is None:
+            logger.debug('Cannot determine appropriate slicing.')
+            raise NotImplementedError
+        x, y = self.get_xy_from_lonlat(np.rad2deg(intersection.lon),
+                                       np.rad2deg(intersection.lat))
+
+        return (slice(np.ma.min(x), np.ma.max(x) + 1),
+                slice(np.ma.min(y), np.ma.max(y) + 1))
+
+    def crop_around(self, other_area):
+        """Crop this area around `other_area`."""
+        xslice, yslice = self.get_area_slices(other_area)
+        return self[yslice, xslice]
+
+    def __getitem__(self, key):
+        """Apply slices to the area_extent and size of the area."""
+        yslice, xslice = key
+        new_area_extent = ((self.pixel_upper_left[0] +
+                            (xslice.start - 0.5) * self.pixel_size_x),
+                           (self.pixel_upper_left[1] -
+                            (yslice.stop - 0.5) * self.pixel_size_y),
+                           (self.pixel_upper_left[0] +
+                            (xslice.stop - 0.5) * self.pixel_size_x),
+                           (self.pixel_upper_left[1] -
+                            (yslice.start - 0.5) * self.pixel_size_y))
+
+        new_area = AreaDefinition(self.area_id, self.name,
+                                  self.proj_id, self.proj_dict,
+                                  xslice.stop - xslice.start,
+                                  yslice.stop - yslice.start,
+                                  new_area_extent)
+        new_area.crop_offset = (self.crop_offset[0] + yslice.start,
+                                self.crop_offset[1] + xslice.start)
+        return new_area
+
+
+def get_geostationary_angle_extent(geos_area):
+    """Get the max earth (vs space) viewing angles in x and y."""
+
+    # get some projection parameters
+    req = geos_area.proj_dict['a'] / 1000
+    rp = geos_area.proj_dict['b'] / 1000
+    h = geos_area.proj_dict['h'] / 1000 + req
+
+    # compute some constants
+    aeq = 1 - req**2 / (h ** 2)
+    ap_ = 1 - rp**2 / (h ** 2)
+
+    # generate points around the north hemisphere in satellite projection
+    # make it a bit smaller so that we stay inside the valid area
+    xmax = np.arccos(np.sqrt(aeq))
+    ymax = np.arccos(np.sqrt(ap_))
+    return xmax, ymax
+
+
+def get_geostationary_bounding_box(geos_area, nb_points=50):
+    """Get the bbox in lon/lats of the valid pixels inside `geos_area`.
+
+    Args:
+      nb_points: Number of points on the polygon
+    """
+    xmax, ymax = get_geostationary_angle_extent(geos_area)
+
+    # generate points around the north hemisphere in satellite projection
+    # make it a bit smaller so that we stay inside the valid area
+    x = np.cos(np.linspace(-np.pi, 0, int(nb_points / 2))) * (xmax - 0.0001)
+    y = -np.sin(np.linspace(-np.pi, 0, int(nb_points / 2))) * (ymax - 0.0001)
+
+    ll_x, ll_y, ur_x, ur_y = geos_area.area_extent
+
+    x *= geos_area.proj_dict['h']
+    y *= geos_area.proj_dict['h']
+
+    x = np.clip(np.concatenate([x, x[::-1]]), min(ll_x, ur_x), max(ll_x, ur_x))
+    y = np.clip(np.concatenate([y, -y]), min(ll_y, ur_y), max(ll_y, ur_y))
+
+    return Proj(**geos_area.proj_dict)(x, y, inverse=True)
 
 
 def combine_area_extents_vertical(area1, area2):
@@ -1345,7 +1468,7 @@ def combine_area_extents_vertical(area1, area2):
 
 
 def concatenate_area_defs(area1, area2, axis=0):
-    """Append *area2* to *area1* and return the results"""
+    """Append *area2* to *area1* and return the results."""
     different_items = (set(area1.proj_dict.items()) ^
                        set(area2.proj_dict.items()))
     if axis == 0:
@@ -1472,7 +1595,19 @@ class StackedAreaDefinition(BaseDefinition):
     @property
     def proj4_string(self):
         """Returns projection definition as Proj.4 string"""
-        return self.defs[0].proj4_string
+        warnings.warn("'proj4_string' is deprecated, please use 'proj_str' "
+                      "instead.", DeprecationWarning)
+        return self.defs[0].proj_str
+
+    @property
+    def proj_str(self):
+        """Returns projection definition as Proj.4 string"""
+        return self.defs[0].proj_str
+
+    def update_hash(self, the_hash=None):
+        for areadef in self.defs:
+            the_hash = areadef.update_hash(the_hash)
+        return the_hash
 
 
 def _get_slice(segments, shape):
